/*------------------------------------------------------------------------------
* rtkcmn.c : rtk common functions
*
*          Copyright (C) 2007-2009 by T.TAKASU, All rights reserved.
*
* options : -DLAPACK   use LAPACK/BLAS
*           -DMKL      use Intel MKL
*           -DTRACE    enable debug trace
*           -DWIN32    use WIN32 API
*
* version : $Revision: 1.1 $ $Date: 2008/07/17 21:48:06 $
* history : 2007/01/12 1.0 new
*           2007/03/06 1.1 input initial rover pos of pntpos()
*                          update only effective states of filter()
*                          fix bug of atan2() domain error
*           2007/04/11 1.2 add function antmodel()
*                          add gdop mask for pntpos()
*                          change constant MAXDTOE value
*           2007/05/25 1.3 add function execcmd(),expandpath()
*           2008/06/21 1.4 add funciton sortobs(),uniqeph(),screent()
*                          replace geodist() by sagnac correction way
*           2008/10/29 1.5 fix bug of ionosphereic mapping function
*                          fix bug of seasonal variation term of tropmapf
*           2008/12/27 1.6 add function tickget(), sleepms(), tracenav(),
*                          xyz2enu(), satposv(), pntvel(), covecef()
*           2009/03/12 1.7 fix bug on error-stop when localtime() returns NULL
*           2009/03/13 1.8 fix bug on time adjustment for summer time
*           2009/04/10 1.9 add function adjgpsweek(),getbits(),getbitu()
*                          add function geph2pos()
*-----------------------------------------------------------------------------*/
#ifdef WIN32
#include <windows.h>
#else
#include <dirent.h>
#include <sys/time.h>
#endif
#include <stdarg.h>
#include <ctype.h>
#include "rtklib.h"
#ifdef MKL
#include <mkl_types.h>
#include <mkl_lapack.h>
#include <mkl_blas.h>
#endif

static const char rcsid[]="$Id: rtkcmn.c,v 1.1 2008/07/17 21:48:06 ttaka Exp ttaka $";

/* constants -----------------------------------------------------------------*/

#define MAXITR      10          /* max number of iteration for point pos */
#define ERRA        1.5         /* measurement error factor a of code */
#define ERRB        1.0         /* measurement error factor b of code */
#define ERRI        0.3         /* ionospheric model error factor */
#define RE_GLO      6378136.0   /* radius of earth (GLONASS-ICD) (m) */
#define MU_GLO      3.9860044E14 /* gravitational constant (GLONASS-ICD) (m^3/s^2) */
#define J2_GLO      1.0826257E-3 /* 2nd zonal harmonic of geop (GLONASS-ICD) */
#define OMGE_GLO    7.292115E-5 /* earth angular velocity (GLONASS-ICD) (rad/s) */
#define TSTEP       30.0        /* integration step for GLONASS ephemeris (s) */

#define POLYCRC32   0xEDB88320u /* CRC32 polynomial */
#define POLYCRC24Q  0x1864CFBu  /* CRC24Q polynomial */

const static double gpst0[]={1980,1,6,0,0,0}; /* gpsweek 0 - day 1 */

const static double leaps[][7]={ /* leap seconds {y,m,d,h,m,s,utc-gpst,...} */
    {2009,1,1,0,0,0,-15},
    {2006,1,1,0,0,0,-14},
    {1999,1,1,0,0,0,-13},
    {1997,7,1,0,0,0,-12},
    {1996,1,1,0,0,0,-11},
    {1994,7,1,0,0,0,-10},
    {1993,7,1,0,0,0, -9},
    {1992,7,1,0,0,0, -8},
    {1991,1,1,0,0,0, -7},
    {1990,1,1,0,0,0, -6},
    {1988,1,1,0,0,0, -5},
    {1985,7,1,0,0,0, -4},
    {1983,7,1,0,0,0, -3},
    {1982,7,1,0,0,0, -2},
    {1981,7,1,0,0,0, -1}
};
const double chisqr[45]={        /* chi-sqr(n) (alpha=0.001) */
    10.8,13.8,16.3,18.5,20.5,22.5,24.3,26.1,27.9,29.6,31.3,32.9,34.5,36.1,37.7,
    39.3,40.8,42.3,43.8,45.3,46.8,48.3,49.7,51.2,52.6,54.1,55.5,56.9,58.3,59.7,
    61.1,62.5,63.9,65.2,66.6,68.0,69.3,70.7,70.1,73.4,86.7,99.6,112.3,124.8,137.2
};
const double lam[]={             /* carrier wave length (m) */
    CLIGHT/FREQ1,CLIGHT/FREQ2,CLIGHT/FREQ5,CLIGHT/FREQE5b,CLIGHT/FREQE6
};
const prcopt_t prcopt_default={  /* defaults processing options */
    0,2,15.0*D2R,0.0,0,0,5,0,0,0,1,0,0,0,0,
    {100.0,0.003,0.003,0.000},{10.0,0.02,0.02},
    {1E-4,1E-3,1E-4},5E-12,3.0,0.0,0.05,30.0,30.0,{0},{0},NULL,NULL,{0}
};
const solopt_t solopt_default={  /* defaults solution output options */
    0,0,1,3,0,1,0,0,0,0," "
};
const char *rcvnamelist[]={ /* receiver name list */
    "RTCM 2","RTCM 3","NovAtel OEM4/V","NovAtel OEM3","u-blox","Superstar II",
    "Crescent",NULL
};
/* lapack/blas prototypes ----------------------------------------------------*/
#ifdef LAPACK
extern void dgemm_(char *, char *, int *, int *, int *, double *, double *,
                   int *, double *, int *, double *, double *, int *);
extern void dgetrf_(int *, int *, double *, int *, int *, int *);
extern void dgetri_(int *, double *, int *, int *, double *, int *, int *);
extern void dgetrs_(char *, int *, int *, double *, int *, int *, double *,
                    int *, int *);
#endif
#ifdef MKL
#define LAPACK
#define dgemm_      dgemm
#define dgetrf_     dgetrf
#define dgetri_     dgetri
#define dgetrs_     dgetrs
#endif

/* fatal error ---------------------------------------------------------------*/
static void fatalerr(const char *format, ...)
{
    va_list ap;
    va_start(ap,format); vfprintf(stderr,format,ap); va_end(ap);
    exit(-9);
}
/* satellite system+prn/slot number to satellite number ------------------------
* convert satellite system+prn/slot number to satellite number
* args   : int    sys       I   satellite system (SYS_GPS,SYS_GLO,...)
*          int    prn       I   satellite prn/slot number
* return : satellite number (0:error)
*-----------------------------------------------------------------------------*/
extern int satno(int sys, int prn)
{
    if (prn<=0) return 0;
    switch (sys) {
        case SYS_GPS: return prn>MAXPRNGPS?0:prn;
        case SYS_GLO: return prn>MAXPRNGLO?0:prn+MAXPRNGPS;
        case SYS_GAL: return prn>MAXPRNGAL?0:prn+MAXPRNGPS+MAXPRNGLO;
        case SYS_QZS: return prn>MAXPRNGAL?0:prn+MAXPRNGPS+MAXPRNGLO+MAXPRNGAL;
        case SYS_SBS:
            return prn<MINPRNSBS||MAXPRNSBS<prn?0:
                   prn+MAXPRNGPS+MAXPRNGLO+MAXPRNGAL+MAXPRNQZS-MINPRNSBS+1;
    }
    return 0;
}
/* satellite number to satellite system ----------------------------------------
* convert satellite number to satellite system
* args   : int    sat       I   satellite number (1-MAXSAT)
*          int    *prn      IO  satellite prn/slot number (NULL: no output)
* return : satellite system (SYS_GPS,SYS_GLO,...)
*-----------------------------------------------------------------------------*/
extern int satsys(int sat, int *prn)
{
    int sys=SYS_NONE;
    if (sat<=0||MAXSAT<sat) sat=0;
    else if ( sat            <=MAXPRNGPS) sys=SYS_GPS;
    else if ((sat-=MAXPRNGPS)<=MAXPRNGLO) sys=SYS_GLO;
    else if ((sat-=MAXPRNGLO)<=MAXPRNGAL) sys=SYS_GAL;
    else if ((sat-=MAXPRNGAL)<=MAXPRNQZS) sys=SYS_QZS;
    else if ((sat-=MAXPRNQZS)<=MAXPRNSBS-MINPRNSBS+1) {
        sat+=MINPRNSBS-1; sys=SYS_SBS;
    }
    else sat=0;
    if (prn) *prn=sat;
    return sys;
}
/* satellite id to satellite number --------------------------------------------
* convert satellite id to satellite number
* args   : char   *id       I   satellite id (nn,Gnn,Rnn,Enn,Qnn or Snn)
* return : satellite number (0: error)
*-----------------------------------------------------------------------------*/
extern int satid2no(const char *id)
{
    int sys,prn;
    char code;
    
    if (sscanf(id,"%d",&prn)==1) {
        if (1<=prn&&prn<=MAXPRNGPS) sys=SYS_GPS;
        else if (MINPRNSBS<=prn&&prn<=MAXPRNSBS) sys=SYS_SBS;
        else return 0;
        return satno(sys,prn);
    }
    if (sscanf(id,"%c%d",&code,&prn)<2) return 0;
    
    switch (code) {
        case 'G': sys=SYS_GPS; break;
        case 'R': sys=SYS_GLO; break;
        case 'E': sys=SYS_GAL; break;
        case 'Q': sys=SYS_QZS; break;
        case 'S': sys=SYS_SBS; prn+=100; break;
        default: return 0;
    }
    return satno(sys,prn);
}
/* satellite number to satellite id --------------------------------------------
* convert satellite number to satellite id
* args   : int    sat       I   satellite number
           char   *id       O   satellite id ( nn,Gnn,Rnn,Enn,Qnn or nnn)
* return : none
*-----------------------------------------------------------------------------*/
extern void satno2id(int sat, char *id)
{
    int prn;
    switch (satsys(sat,&prn)) {
        case SYS_GPS: sprintf(id,"%d" ,prn); return;
        case SYS_GLO: sprintf(id,"R%d",prn); return;
        case SYS_GAL: sprintf(id,"E%d",prn); return;
        case SYS_QZS: sprintf(id,"Q%d",prn); return;
        case SYS_SBS: sprintf(id,"%d" ,prn); return;
    }
    strcpy(id,"");
}
/* extract unsigned/signed bits ------------------------------------------------
* extract unsigned bits from byte data
* args   : unsigned char *buff I byte data
*          int    pos    I      bit position from start of data (bits)
*          int    len    I      bit length (bits) (<=32)
* return : extracted unsigned/signed bits
*-----------------------------------------------------------------------------*/
extern unsigned int getbitu(const unsigned char *buff, int pos, int len)
{
    unsigned int bits=0;
    int i;
    for (i=pos;i<pos+len;i++) bits=(bits<<1)+((buff[i/8]>>(7-i%8))&1u);
    return bits;
}
extern int getbits(const unsigned char *buff, int pos, int len)
{
    unsigned int bits=getbitu(buff,pos,len);
    if (len<=0||32<=len||!(bits&(1u<<(len-1)))) return (int)bits;
    return (int)(bits|(~0u<<len)); /* extend sign */
}

/* crc-32 parity ---------------------------------------------------------------
* compute crc-32 parity for novatel raw
* args   : unsigned char *buff I data
*          int    len    I      data length (bytes)
* return : crc-32 parity
* notes  : see NovAtel OEMV firmware manual 1.7 32-bit CRC
*-----------------------------------------------------------------------------*/
extern unsigned int crc32(const unsigned char *buff, int len)
{
    int i,j;
    unsigned int crc=0;
    
    trace(4,"crc32: len=%d\n",len);
    
    for (i=0;i<len;i++) {
        crc^=buff[i];
        for (j=0;j<8;j++) {
            if (crc&1) crc=(crc>>1)^POLYCRC32; else crc>>=1;
        }
    }
    return crc;
}
/* crc-24q parity --------------------------------------------------------------
* compute crc-24q parity for sbas, rtcm3
* args   : unsigned char *buff I data
*          int    len    I      data length (bytes)
* return : crc-24Q parity
* notes  : see RTCA/DO-229C A.4.3.3 Parity
*-----------------------------------------------------------------------------*/
extern unsigned int crc24q(const unsigned char *buff, int len)
{
    static unsigned int crcs[256]={0}; /* crc table */
    int i,j;
    unsigned int crc=0;
    
    trace(4,"crc24q: len=%d\n",len);
    
    if (!crcs[1]) { /* generate table */
        for (i=0;i<256;i++) {
            crcs[i]=(unsigned int)i<<16;
            for (j=0;j<8;j++) if ((crcs[i]<<=1)&0x1000000) crcs[i]^=POLYCRC24Q;
        }
    }
    for (i=0;i<len;i++) crc=((crc<<8)&0xFFFFFF)^crcs[(crc>>16)^buff[i]];
    return crc;
}
/* new matrix ------------------------------------------------------------------
* allocate memory of matrix 
* args   : int    n,m       I   number of rows and columns of matrix
* return : matrix pointer (if n<=0 or m<=0, return NULL)
*-----------------------------------------------------------------------------*/
extern double *mat(int n, int m)
{
    double *p;
    
    if (n<=0||m<=0) return NULL;
    if (!(p=(double *)malloc(sizeof(double)*n*m))) {
        fatalerr("matrix memory allocation error: n=%d,m=%d\n",n,m);
    }
    return p;
}
/* new integer matrix ----------------------------------------------------------
* allocate memory of integer matrix 
* args   : int    n,m       I   number of rows and columns of matrix
* return : matrix pointer (if n<=0 or m<=0, return NULL)
*-----------------------------------------------------------------------------*/
extern int *imat(int n, int m)
{
    int *p;
    
    if (n<=0||m<=0) return NULL;
    if (!(p=(int *)malloc(sizeof(int)*n*m))) {
        fatalerr("integer matrix memory allocation error: n=%d,m=%d\n",n,m);
    }
    return p;
}
/* zero matrix -----------------------------------------------------------------
* generate new zero matrix
* args   : int    n,m       I   number of rows and columns of matrix
* return : matrix pointer (if n<=0 or m<=0, return NULL)
*-----------------------------------------------------------------------------*/
extern double *zeros(int n, int m)
{
    double *p;
    
    if ((p=mat(n,m))) for (n=n*m-1;n>=0;n--) p[n]=0.0;
    return p;
}
/* identity matrix -------------------------------------------------------------
* generate new identity matrix
* args   : int    n         I   number of rows and columns of matrix
* return : matrix pointer (if n<=0, return NULL)
*-----------------------------------------------------------------------------*/
extern double *eye(int n)
{
    double *p;
    int i;
    
    if ((p=zeros(n,n))) for (i=0;i<n;i++) p[i+i*n]=1.0;
    return p;
}
/* inner product ---------------------------------------------------------------
* inner product of vectors
* args   : double a,b       I   vector a,b (n x 1)
*          int    n         I   size of vector a,b
* return : a'*b
*-----------------------------------------------------------------------------*/
extern double dot(const double *a, const double *b, int n)
{
    double c=0.0;
    
    while (--n>=0) c+=a[n]*b[n];
    return c;
}
/* euclid norm -----------------------------------------------------------------
* euclid norm of vector
* args   : double *a        I   vector a (n x 1)
*          int    n         I   size of vector a
* return : || a ||
*-----------------------------------------------------------------------------*/
extern double norm(const double *a, int n)
{
    return sqrt(dot(a,a,n));
}
/* copy matrix -----------------------------------------------------------------
* copy matrix
* args   : double *A        O   destination matrix A (n x m)
*          double *B        I   source matrix B (n x m)
*          int    n,m       I   number of rows and columns of matrix
* return : none
*-----------------------------------------------------------------------------*/
extern void matcpy(double *A, const double *B, int n, int m)
{
    memcpy(A,B,sizeof(double)*n*m);
}
/* matrix routines -----------------------------------------------------------*/

#ifdef LAPACK /* with LAPACK/BLAS or MKL */

/* multiply matrix (wrapper of blas dgemm) -------------------------------------
* multiply matrix by matrix (C=alpha*A*B+beta*C)
* args   : char   *tr       I  transpose flags ("N":normal,"T":transpose)
*          int    n,k,m     I  size of (transposed) matrix A,B
*          double alpha     I  alpha
*          double *A,*B     I  (transposed) matrix A (n x m), B (m x k)
*          double beta      I  beta
*          double *C        IO matrix C (n x k)
* return : none
*-----------------------------------------------------------------------------*/
extern void matmul(const char *tr, int n, int k, int m, double alpha,
                   const double *A, const double *B, double beta, double *C)
{
    int lda=tr[0]=='T'?m:n,ldb=tr[1]=='T'?k:m;
    
    dgemm_((char *)tr,(char *)tr+1,&n,&k,&m,&alpha,(double *)A,&lda,(double *)B,
           &ldb,&beta,C,&n);
}
/* inverse of matrix -----------------------------------------------------------
* inverse of matrix (A=A^-1)
* args   : double *A        IO  matrix (n x n)
*          int    n         I   size of matrix A
* return : status (0:ok,0>:error)
*-----------------------------------------------------------------------------*/
extern int matinv(double *A, int n)
{
    double *work;
    int info,lwork=n*16,*ipiv=imat(n,1);
    
    work=mat(lwork,1);
    dgetrf_(&n,&n,A,&n,ipiv,&info);
    if (!info) dgetri_(&n,A,&n,ipiv,work,&lwork,&info);
    free(ipiv); free(work);
    return info;
}
/* solve linear equation -------------------------------------------------------
* solve linear equation (X=A\Y or X=A'\Y)
* args   : char   *tr       I   transpose flag ("N":normal,"T":transpose)
*          double *A        I   input matrix A (n x n)
*          double *Y        I   input matrix Y (n x m)
*          int    n,m       I   size of matrix A,Y
*          double *X        O   X=A\Y or X=A'\Y (n x m)
* return : status (0:ok,0>:error)
* notes  : matirix stored by column-major order (fortran convention)
*          X can be same as Y
*-----------------------------------------------------------------------------*/
extern int solve(const char *tr, const double *A, const double *Y, int n,
                 int m, double *X)
{
    double *B=mat(n,n);
    int info,*ipiv=imat(n,1);
    
    matcpy(B,A,n,n);
    matcpy(X,Y,n,m);
    dgetrf_(&n,&n,B,&n,ipiv,&info);
    if (!info) dgetrs_((char *)tr,&n,&m,B,&n,ipiv,X,&n,&info);
    free(ipiv); free(B); 
    return info;
}

#else /* without LAPACK/BLAS or MKL */

/* multiply matrix -----------------------------------------------------------*/
extern void matmul(const char *tr, int n, int k, int m, double alpha,
                   const double *A, const double *B, double beta, double *C)
{
    double d;
    int i,j,x,f=tr[0]=='N'?(tr[1]=='N'?1:2):(tr[1]=='N'?3:4);
    
    for (i=0;i<n;i++) for (j=0;j<k;j++) {
        d=0.0;
        switch (f) {
        case 1: for (x=0;x<m;x++) d+=A[i+x*n]*B[x+j*m]; break;
        case 2: for (x=0;x<m;x++) d+=A[i+x*n]*B[j+x*k]; break;
        case 3: for (x=0;x<m;x++) d+=A[x+i*m]*B[x+j*m]; break;
        case 4: for (x=0;x<m;x++) d+=A[x+i*m]*B[j+x*k]; break;
        }
        if (beta==0.0) C[i+j*n]=alpha*d; else C[i+j*n]=alpha*d+beta*C[i+j*n];
    }
}
/* LU decomposition ----------------------------------------------------------*/
static int ludcmp(double *A, int n, int *indx, double *d)
{
    double big,s,tmp,*vv=mat(n,1);
    int i,imax=0,j,k;
    
    *d=1.0;
    for (i=0;i<n;i++) {
        big=0.0; for (j=0;j<n;j++) if ((tmp=fabs(A[i+j*n]))>big) big=tmp;
        if (big>0.0) vv[i]=1.0/big; else {free(vv); return -1;}
    }
    for (j=0;j<n;j++) {
        for (i=0;i<j;i++) {
            s=A[i+j*n]; for (k=0;k<i;k++) s-=A[i+k*n]*A[k+j*n]; A[i+j*n]=s;
        }
        big=0.0;
        for (i=j;i<n;i++) {
            s=A[i+j*n]; for (k=0;k<j;k++) s-=A[i+k*n]*A[k+j*n]; A[i+j*n]=s;
            if ((tmp=vv[i]*fabs(s))>=big) {big=tmp; imax=i;}
        }
        if (j!=imax) {
            for (k=0;k<n;k++) {
                tmp=A[imax+k*n]; A[imax+k*n]=A[j+k*n]; A[j+k*n]=tmp;
            }
            *d=-(*d); vv[imax]=vv[j];
        }
        indx[j]=imax;
        if (A[j+j*n]==0.0) {free(vv); return -1;}
        if (fabs(A[j+j*n])<1E-16) {free(vv); return -1;}
        if (j!=n-1) {
            tmp=1.0/A[j+j*n]; for (i=j+1;i<n;i++) A[i+j*n]*=tmp;
        }
    }
    free(vv);
    return 0;
}
/* LU back-substitution ------------------------------------------------------*/
static void lubksb(const double *A, int n, const int *indx, double *b)
{
    double s;
    int i,ii=-1,ip,j;
    
    for (i=0;i<n;i++) {
        ip=indx[i]; s=b[ip]; b[ip]=b[i];
        if (ii>=0) for (j=ii;j<i;j++) s-=A[i+j*n]*b[j]; else if (s) ii=i;
        b[i]=s;
    }
    for (i=n-1;i>=0;i--) {
        s=b[i]; for (j=i+1;j<n;j++) s-=A[i+j*n]*b[j]; b[i]=s/A[i+i*n];
    }
}
/* inverse of matrix ---------------------------------------------------------*/
extern int matinv(double *A, int n)
{
    double d,*B;
    int i,j,*indx;
    
    indx=imat(n,1); B=mat(n,n); matcpy(B,A,n,n);
    if (ludcmp(B,n,indx,&d)) {free(indx); free(B); return -1;}
    for (j=0;j<n;j++) {
        for (i=0;i<n;i++) A[i+j*n]=0.0; A[j+j*n]=1.0;
        lubksb(B,n,indx,A+j*n);
    }
    free(indx); free(B);
    return 0;
}
/* solve linear equation -----------------------------------------------------*/
extern int solve(const char *tr, const double *A, const double *Y, int n,
                 int m, double *X)
{
    double *B=mat(n,n);
    int info;
    
    matcpy(B,A,n,n);
    if (!(info=matinv(B,n))) matmul(tr[0]=='N'?"NN":"TN",n,m,n,1.0,B,Y,0.0,X);
    free(B);
    return info;
}
#endif
/* end of matrix routines ----------------------------------------------------*/

/* least square estimation -----------------------------------------------------
* least square estimation by solving normal equation (x=(A*A')^-1*A*y)
* args   : double *A        I   transpose of (weighted) design matrix (n x m)
*          double *y        I   (weighted) measurements (m x 1)
*          int    n,m       I   number of parameters and measurements (n<=m)
*          double *x        O   estmated parameters (n x 1)
*          double *Q        O   esimated parameters covariance matrix (n x n)
* return : status (0:ok,0>:error)
* notes  : for weighted least square, replace A and y by A*w and w*y (w=W^(1/2))
*          matirix stored by column-major order (fortran convention)
*-----------------------------------------------------------------------------*/
extern int lsq(const double *A, const double *y, int n, int m, double *x,
               double *Q)
{
    double *Ay;
    int info;
    
    if (m<n) return -1;
    Ay=mat(n,1);
    matmul("NN",n,1,m,1.0,A,y,0.0,Ay); /* Ay=A*y */
    matmul("NT",n,n,m,1.0,A,A,0.0,Q);  /* Q=A*A' */
    if (!(info=matinv(Q,n))) matmul("NN",n,1,n,1.0,Q,Ay,0.0,x); /* x=Q^-1*Ay */
    free(Ay);
    return info;
}
/* kalman filter ---------------------------------------------------------------
* kalman filter state update (K=P*H*(H'*P*H+R)^-1,xp=x+K*v,Pp=(I-K*H')*P)
* args   : double *x        IO  states vector (n x 1)
*          double *P        IO  covariance matrix of states (n x n)
*          double *H        I   transpose of design matrix (n x m)
*          double *v        I   innovation (measurement - model) (m x 1)
*          double *R        I   covariance matrix of measurement error (m x m)
*          int    n,m       I   number of states and measurements
* return : status (0:ok,<0:error)
* notes  : matirix stored by column-major order (fortran convention)
*          if state x[i]==0.0, not updates state x[i]/P[i+i*n]
*-----------------------------------------------------------------------------*/
static int filter_(const double *x, const double *P, const double *H,
                   const double *v, const double *R, int n, int m,
                   double *xp, double *Pp)
{
    double *F=mat(n,m),*Q=mat(m,m),*K=mat(n,m),*I=eye(n);
    int info;
    
    matcpy(Q,R,m,m);
    matcpy(xp,x,n,1);
    matmul("NN",n,m,n,1.0,P,H,0.0,F);       /* Q=H'*P*H+R */
    matmul("TN",m,m,n,1.0,H,F,1.0,Q);
    if (!(info=matinv(Q,m))) {
        matmul("NN",n,m,m,1.0,F,Q,0.0,K);   /* K=P*H*Q^-1 */
        matmul("NN",n,1,m,1.0,K,v,1.0,xp);  /* xp=x+K*v */
        matmul("NT",n,n,m,-1.0,K,H,1.0,I);  /* Pp=(I-K*H')*P */
        matmul("NN",n,n,n,1.0,I,P,0.0,Pp);
    }
    free(F); free(Q); free(K); free(I);
    return info;
}
extern int filter(double *x, double *P, const double *H, const double *v,
                  const double *R, int n, int m)
{
    double *x_,*xp_,*P_,*Pp_,*H_;
    int i,j,k,info,*ix;
    
    ix=imat(n,1); for (i=k=0;i<n;i++) if (x[i]!=0.0&&P[i+i*n]>0.0) ix[k++]=i;
    x_=mat(k,1); xp_=mat(k,1); P_=mat(k,k); Pp_=mat(k,k); H_=mat(k,m);
    for (i=0;i<k;i++) {
        x_[i]=x[ix[i]];
        for (j=0;j<k;j++) P_[i+j*k]=P[ix[i]+ix[j]*n];
        for (j=0;j<m;j++) H_[i+j*k]=H[ix[i]+j*n];
    }
    info=filter_(x_,P_,H_,v,R,k,m,xp_,Pp_);
    for (i=0;i<k;i++) {
        x[ix[i]]=xp_[i];
        for (j=0;j<k;j++) P[ix[i]+ix[j]*n]=Pp_[i+j*k];
    }
    free(ix); free(x_); free(xp_); free(P_); free(Pp_); free(H_);
    return info;
}
/* smoother --------------------------------------------------------------------
* combine forward and backward filters by fixed-interval smoother (ref[1], 5.2)
* (xs=Qs*(Qf^-1*xf+Qb^-1*xb),Qs=(Qf^-1+Qb^-1)^-1)
* args   : double *xf,*Qf   I   forward solutions/covariances  (n x 1/n x n)
*          double *xb,*Qb   I   backward solutions/covariances (n x 1/n x n)
*          int    n         I   number of solutions
*          double *xs,*Qs   O   smoothed solutions/covariances (n x 1/n x n)
* return : status (0:ok,0>:error)
* notes  : reference :
*          [1] A.Gelb ed., Applied Optimal Estimation, The M.I.T Press, 1974
*          matirix stored by column-major order (fortran convention)
*-----------------------------------------------------------------------------*/
extern int smoother(const double *xf, const double *Qf, const double *xb,
                    const double *Qb, int n, double *xs, double *Qs)
{
    double *invQf=mat(n,n),*invQb=mat(n,n),*xx=mat(n,1);
    int i,info=-1;
    
    matcpy(invQf,Qf,n,n);
    matcpy(invQb,Qb,n,n);
    if (!matinv(invQf,n)&&!matinv(invQb,n)) {
        for (i=0;i<n*n;i++) Qs[i]=invQf[i]+invQb[i];
        if (!(info=matinv(Qs,n))) {
            matmul("NN",n,1,n,1.0,invQf,xf,0.0,xx);
            matmul("NN",n,1,n,1.0,invQb,xb,1.0,xx);
            matmul("NN",n,1,n,1.0,Qs,xx,0.0,xs);
        }
    }
    free(invQf); free(invQb); free(xx);
    return info;
}
/* print matrix ----------------------------------------------------------------
* print matrix to stdout
* args   : double *A        I   matrix A (n x m)
*          int    n,m       I   number of rows and columns of A
*          int    p,q       I   total columns, columns under decimal point
*         (FILE  *fp        I   output file pointer)
* return : none
* notes  : matirix stored by column-major order (fortran convention)
*-----------------------------------------------------------------------------*/
extern void matfprint(const double A[], int n, int m, int p, int q, FILE *fp)
{
    int i,j;
    
    for (i=0;i<n;i++) {
        for (j=0;j<m;j++) fprintf(fp," %*.*f",p,q,A[i+j*n]);
        fprintf(fp,"\n");
    }
}
extern void matprint(const double A[], int n, int m, int p, int q)
{
    matfprint(A,n,m,p,q,stdout);
}
/* string to number ------------------------------------------------------------
* convert substring in string to number
* args   : char   *s        I   string ("... nnn.nnn ...")
*          int    i,n       I   substring position and width
* return : converted number (0.0:error)
*-----------------------------------------------------------------------------*/
extern double str2num(const char *s, int i, int n)
{
    double value;
    char str[256],*p=str;
    
    if (i<0||(int)strlen(s)<i||sizeof(str)-1<i) return 0.0;
    for (s+=i;*s&&--n>=0;s++) *p++=*s=='d'||*s=='D'?'E':*s; *p='\0';
    return sscanf(str,"%lf",&value)==1?value:0.0;
}
/* string to time --------------------------------------------------------------
* convert substring in string to gtime_t struct
* args   : char   *s        I   string ("... yyyy mm dd hh mm ss ...")
*          int    i,n       I   substring position and width
*          gtime_t *t       O   gtime_t struct
* return : status (0:ok,0>:error)
*-----------------------------------------------------------------------------*/
extern int str2time(const char *s, int i, int n, gtime_t *t)
{
    double ep[6];
    char str[256],*p=str;
    
    if (i<0||(int)strlen(s)<i||sizeof(str)-1<i) return -1;
    for (s+=i;*s&&--n>=0;) *p++=*s++; *p='\0';
    if (sscanf(str,"%lf %lf %lf %lf %lf %lf",ep,ep+1,ep+2,ep+3,ep+4,ep+5)<6)
        return -1;
    if (ep[0]<100.0) ep[0]+=ep[0]<80.0?2000.0:1900.0;
    *t=epoch2time(ep);
    return 0;
}
/* convert calendar day/time to time -------------------------------------------
* convert calendar day/time to gtime_t struct
* args   : double *ep       I   day/time {year,month,day,hour,min,sec}
* return : gtime_t struct
*-----------------------------------------------------------------------------*/
extern gtime_t epoch2time(const double *ep)
{
    struct tm t={0};
    gtime_t time;
    double sec=ep[5];
    int dsec=0;
    
    if (sec>=60.0) {sec-=60.0; dsec=60;}
    else if (sec<0.0) {sec+=60.0; dsec=-60;}
    t.tm_year=(int)ep[0]-1900; t.tm_mon=(int)ep[1]-1; t.tm_mday=(int)ep[2];
    t.tm_hour=(int)ep[3]; t.tm_min=(int)ep[4]; t.tm_sec=(int)sec;
    time.time=mktime(&t)+dsec; time.sec=sec-t.tm_sec;
    return time;
}
/* adjust summer time --------------------------------------------------------*/
static void adjsummertime(struct tm *tt)
{
    const int days[]={31,28,31,30,31,30,31,31,30,31,30,31};
    
    if (--tt->tm_hour>=0) return;
    tt->tm_hour=23;
    if (--tt->tm_mday>=1) return;
    if (--tt->tm_mon<0) {
        tt->tm_mon=11;
        tt->tm_year--;
    }
    tt->tm_mday=tt->tm_year+1900%4==0&&tt->tm_mon==1?29:days[tt->tm_mon];
}
/* time to calendar day/time ---------------------------------------------------
* convert gtime_t struct to calendar day/time
* args   : gtime_t t        I   gtime_t struct
*          double *ep       O   day/time {year,month,day,hour,min,sec}
* return : none
*-----------------------------------------------------------------------------*/
extern void time2epoch(gtime_t t, double *ep)
{
    struct tm *tt,ttt;
    
    if (!(tt=localtime(&t.time))) {
        ep[0]=1900.0; ep[1]=ep[2]=1.0; ep[3]=ep[4]=ep[5]=0.0;
        return;
    }
    ttt=*tt;
    if (ttt.tm_isdst>0) adjsummertime(&ttt); /* adjust summer time */
    ep[0]=ttt.tm_year+1900; ep[1]=ttt.tm_mon+1; ep[2]=ttt.tm_mday;
    ep[3]=ttt.tm_hour;
    ep[4]=ttt.tm_min; ep[5]=ttt.tm_sec+t.sec;
}
/* gpstime to time -------------------------------------------------------------
* convert gps week and tow to gtime_t struct
* args   : int    week      I   gps week number
*          double sec       I   gpstime (time of week) (sec)
* return : gtime_t struct
*-----------------------------------------------------------------------------*/
extern gtime_t gpst2time(int week, double sec)
{
    gtime_t t=epoch2time(gpst0);
    
    if (sec<-1E9||1E9<sec) sec=0.0;
    t.time+=86400*7*week+(int)sec;
    t.sec=sec-(int)sec;
    return t;
}
/* time to gpstime -------------------------------------------------------------
* convert gtime_t struct to gps week and tow
* args   : gtime_t t        I   gtime_t struct
*          int    *week     IO  gps week number (NULL: no output)
* return : gpstime (time of week) (sec)
*-----------------------------------------------------------------------------*/
extern double time2gpst(gtime_t t, int *week)
{
    gtime_t t0=epoch2time(gpst0);
    time_t sec=t.time-t0.time;
    int w=(int)(sec/(86400*7));
    
    if (week) *week=w;
    return (double)(sec-w*86400*7)+t.sec;
}
/* add time --------------------------------------------------------------------
* add time to gtime_t struct
* args   : gtime_t t        I   gtime_t struct
*          double sec       I   time to add (sec)
* return : gtime_t struct (t+sec)
*-----------------------------------------------------------------------------*/
extern gtime_t timeadd(gtime_t t, double sec)
{
    double tt;
    
    t.sec+=sec; tt=floor(t.sec); t.time+=(int)tt; t.sec-=tt;
    return t;
}
/* time difference -------------------------------------------------------------
* difference between gtime_t structs
* args   : gtime_t t1,t2    I   gtime_t structs
* return : time difference (t1-t2) (sec)
*-----------------------------------------------------------------------------*/
extern double timediff(gtime_t t1, gtime_t t2)
{
    return difftime(t1.time,t2.time)+t1.sec-t2.sec;
}
/* get current time in utc -----------------------------------------------------
* get current time in utc
* args   : none
* return : current time in utc
*-----------------------------------------------------------------------------*/
extern gtime_t timeget(void)
{
    time_t tc=time(NULL); /* calender time */
    struct tm *tt=gmtime(&tc); /* utc */
    double ep[6];
    
    ep[0]=tt->tm_year+1900; ep[1]=tt->tm_mon+1; ep[2]=tt->tm_mday;
    ep[3]=tt->tm_hour; ep[4]=tt->tm_min; ep[5]=tt->tm_sec;
    return epoch2time(ep);
}
/* gpstime to utc --------------------------------------------------------------
* convert gpstime to utc considering leap seconds
* args   : gtime_t t        I   time expressed in gpstime
* return : time expressed in utc
*-----------------------------------------------------------------------------*/
extern gtime_t gpst2utc(gtime_t t)
{
    gtime_t tu;
    int i;
    
    for (i=0;i<sizeof(leaps)/sizeof(*leaps);i++) {
        tu=timeadd(t,leaps[i][6]);
        if (timediff(tu,epoch2time(leaps[i]))>=0.0) return tu;
    }
    return t;
}
/* utc to gpstime --------------------------------------------------------------
* convert utc to gpstime considering leap seconds
* args   : gtime_t t        I   time expressed in utc
* return : time expressed in gpstime
*-----------------------------------------------------------------------------*/
extern gtime_t utc2gpst(gtime_t t)
{
    int i;
    
    for (i=0;i<sizeof(leaps)/sizeof(*leaps);i++) {
        if (timediff(t,epoch2time(leaps[i]))>=0.0) return timeadd(t,-leaps[i][6]);
    }
    return t;
}
/* time to string --------------------------------------------------------------
* convert gtime_t struct to string
* args   : gtime_t t        I   gtime_t struct
*          char   *s        O   string ("yyyy/mm/dd hh:mm:ss.s")
*          int    n         I   columns of sec under decimal point
* return : none
*-----------------------------------------------------------------------------*/
extern void time2str(gtime_t t, char *s, int n)
{
    double ep[6];
    
    if (n>=0&&1.0-t.sec<0.5/pow(10.0,n)) {t.time++; t.sec=0.0;};
    time2epoch(t,ep);
    sprintf(s,"%04.0f/%02.0f/%02.0f %02.0f:%02.0f:%0*.*f",ep[0],ep[1],ep[2],
            ep[3],ep[4],n<=0?2:n+3,n<=0?0:n,ep[5]);
}
/* time to day of year ---------------------------------------------------------
* convert time to day of year
* args   : gtime_t t        I   gtime_t struct
* return : day of year (days)
*-----------------------------------------------------------------------------*/
extern double time2doy(gtime_t t)
{
    double ep[6];
    
    time2epoch(t,ep);
    ep[1]=ep[2]=1.0; ep[3]=ep[4]=ep[5]=0.0;
    return timediff(t,epoch2time(ep))/86400.0+1.0;
}
/* adjust gps week number ------------------------------------------------------
* adjust gps week number using cpu time
* args   : int   week       I   not-adjusted gps week number
* return : adjusted gps week number
*-----------------------------------------------------------------------------*/
extern int adjgpsweek(int week)
{
    int w;
    (void)time2gpst(utc2gpst(timeget()),&w);
    if (w<0) w=1512; /* use 2009/1/1 if cpu time is invalid */
    return week+(w-week+512)/1024*1024;
}
/* get tick time ---------------------------------------------------------------
* get current tick in ms
* args   : none
* return : current tick in ms
*-----------------------------------------------------------------------------*/
extern unsigned int tickget(void)
{
#ifdef WIN32
    return (unsigned int)timeGetTime();
#else
    struct timeval tv={0};
    gettimeofday(&tv,NULL);
    return tv.tv_sec*1000u+tv.tv_usec/1000u;
#endif
}
/* sleep ms --------------------------------------------------------------------
* sleep ms
* args   : int   ms         I   miliseconds to sleep (<0:no sleep)
* return : none
*-----------------------------------------------------------------------------*/
extern void sleepms(int ms)
{
#ifdef WIN32
    if (ms<5) Sleep(1); else Sleep(ms);
#else
    struct timespec ts;
    if (ms<=0) return;
    ts.tv_sec=(time_t)(ms/1000);
    ts.tv_nsec=(long)(ms%1000*1000000);
    nanosleep(&ts,NULL);
#endif
}
/* transform ecef to geodetic postion ------------------------------------------
* transform ecef position to geodetic position
* args   : double *r        I   ecef position {x,y,z} (m)
*          double *pos      O   geodetic position {lat,lon,h} (rad,m)
* return : none
* notes  : WGS84, ellipsoidal height
*-----------------------------------------------------------------------------*/
extern void ecef2pos(const double *r, double *pos)
{
    double e2=FE*(2.0-FE),r2=dot(r,r,2),z,zk,v=RE,sinp;
    
    for (z=r[2],zk=0.0;fabs(z-zk)>=1E-4;) {
        zk=z;
        sinp=z/sqrt(r2+z*z);
        v=RE/sqrt(1.0-e2*sinp*sinp);
        z=r[2]+v*e2*sinp;
    }
    pos[0]=r2>1E-12?atan(z/sqrt(r2)):(r[2]>0.0?PI/2.0:-PI/2.0);
    pos[1]=r2>1E-12?atan2(r[1],r[0]):0.0;
    pos[2]=sqrt(r2+z*z)-v;
}
/* transform geodetic to ecef position -----------------------------------------
* transform geodetic position to ecef position
* args   : double *pos      I   geodetic position {lat,lon,h} (rad,m)
*          double *r        O   ecef position {x,y,z} (m)
* return : none
* notes  : WGS84, ellipsoidal height
*-----------------------------------------------------------------------------*/
extern void pos2ecef(const double *pos, double *r)
{
    double sinp=sin(pos[0]),cosp=cos(pos[0]),sinl=sin(pos[1]),cosl=cos(pos[1]);
    double e2=FE*(2.0-FE),v=RE/sqrt(1.0-e2*sinp*sinp);
    
    r[0]=(v+pos[2])*cosp*cosl;
    r[1]=(v+pos[2])*cosp*sinl;
    r[2]=(v*(1.0-e2)+pos[2])*sinp;
}
/* ecef to local coordinate transfromation matrix ------------------------------
* compute ecef to local coordinate transfromation matrix
* args   : double *pos      I   geodetic position {lat,lon} (rad)
*          double *E        O   ecef to local coord transformation matrix (3x3)
* return : none
* notes  : matirix stored by column-major order (fortran convention)
*-----------------------------------------------------------------------------*/
extern void xyz2enu(const double *pos, double *E)
{
    double sinp=sin(pos[0]),cosp=cos(pos[0]),sinl=sin(pos[1]),cosl=cos(pos[1]);
    
    E[0]=-sinl;      E[3]=cosl;       E[6]=0.0;
    E[1]=-sinp*cosl; E[4]=-sinp*sinl; E[7]=cosp;
    E[2]=cosp*cosl;  E[5]=cosp*sinl;  E[8]=sinp;
}
/* transform ecef vector to local tangental coordinate -------------------------
* transform ecef vector to local tangental coordinate
* args   : double *pos      I   geodetic position {lat,lon} (rad)
*          double *r        I   vector in ecef coordinate {x,y,z}
*          double *e        O   vector in local tangental coordinate {e,n,u}
* return : none
*-----------------------------------------------------------------------------*/
extern void ecef2enu(const double *pos, const double *r, double *e)
{
    double E[9];
    
    xyz2enu(pos,E);
    matmul("NN",3,1,3,1.0,E,r,0.0,e);
}
/* transform local vector to ecef coordinate -----------------------------------
* transform local tangental coordinate vector to ecef
* args   : double *pos      I   geodetic position {lat,lon} (rad)
*          double *e        I   vector in local tangental coordinate {e,n,u}
*          double *r        O   vector in ecef coordinate {x,y,z}
* return : none
*-----------------------------------------------------------------------------*/
extern void enu2ecef(const double *pos, const double *e, double *r)
{
    double E[9];
    
    xyz2enu(pos,E);
    matmul("TN",3,1,3,1.0,E,e,0.0,r);
}
/* transform covariance to local tangental coordinate --------------------------
* transform ecef covariance to local tangental coordinate
* args   : double *pos      I   geodetic position {lat,lon} (rad)
*          double *P        I   covariance in ecef coordinate
*          double *Q        O   covariance in local tangental coordinate
* return : none
*-----------------------------------------------------------------------------*/
extern void covenu(const double *pos, const double *P, double *Q)
{
    double E[9],EP[9];
    
    xyz2enu(pos,E);
    matmul("NN",3,3,3,1.0,E,P,0.0,EP);
    matmul("NT",3,3,3,1.0,EP,E,0.0,Q);
}
/* transform local enu coordinate covariance to xyz-ecef -----------------------
* transform local enu covariance to xyz-ecef coordinate
* args   : double *pos      I   geodetic position {lat,lon} (rad)
*          double *Q        I   covariance in local enu coordinate
*          double *P        O   covariance in xyz-ecef coordinate
* return : none
*-----------------------------------------------------------------------------*/
extern void covecef(const double *pos, const double *Q, double *P)
{
    double E[9],EQ[9];
    
    xyz2enu(pos,E);
    matmul("TN",3,3,3,1.0,E,Q,0.0,EQ);
    matmul("NN",3,3,3,1.0,EQ,E,0.0,P);
}
/* read antenna phase center parameters ----------------------------------------
* read antenna phase center parameters from igs_pcv format file
* args   : char   *file     I   antenna phase center parameter file 
*          char   *ant      I   antenna model
*          pcv_t *pcv       I   antenna phase center parameters
* return : status (0:ok,0>:error)
*-----------------------------------------------------------------------------*/
static void decodepcv(char *p, int n, double *v)
{
    int i=0;
    for (p=strtok(p," ");p&&i<n;p=strtok(NULL," ")) v[i++]=atof(p)*1E-3;
}
static void swappcv(double *a, double *b)
{
    double tmp=*a; *a=*b; *b=tmp;
}
extern int readpcv(const char *file, const char *ant, pcv_t *pcv)
{
    FILE *fp;
    char buff[256];
    int i,j,n=-1;
    
    trace(3,"readpcv: file=%s ant=%s\n",file,ant);
    
    if (!(fp=fopen(file,"r"))) return -1;
    
    strncpy(pcv->ant,ant,127); pcv->ant[127]='\0';
    
    while (fgets(buff,sizeof(buff),fp)) {
        if (n<0) {if (strstr(buff,ant)) n=0; else continue;}
        switch (n++) {
        case 1: decodepcv(buff, 3,pcv->off[0]);    break;
        case 2: decodepcv(buff,10,pcv->var[0]);    break;
        case 3: decodepcv(buff, 9,pcv->var[0]+10); break;
        case 4: decodepcv(buff, 3,pcv->off[1]);    break;
        case 5: decodepcv(buff,10,pcv->var[1]);    break;
        case 6: decodepcv(buff, 9,pcv->var[1]+10);
            for (i=0;i<2;i++) { /* neu->enu, 90...0->0...90 */
                swappcv(pcv->off[i],pcv->off[i]+1);
                for (j=0;j<9;j++) swappcv(pcv->var[i]+j,pcv->var[i]+18-j);
            }
            fclose(fp);
            return 0;
        }
    }
    strcpy(pcv->ant,"");
    fclose(fp);
    return -1;
}
/* read station positions ------------------------------------------------------
* read positions from station position file
* args   : char  *file      I   station position file containing
*                               lat(deg) lon(deg) height(m) name in a line
*          char  *rcvs      I   station name
*          double *pos      O   station position {lat,lon,h} (rad/m)
*                               (all 0 if search error)
* return : none
*-----------------------------------------------------------------------------*/
extern void readpos(const char *file, const char *rcv, double *pos)
{
    static double poss[2048][3];
    static char stas[2048][16];
    FILE *fp;
    int i,j,len,np=0;
    char buff[256],str[256];
    
    if (!(fp=fopen(file,"r"))) {
        fprintf(stderr,"reference position file open error : %s\n",file);
        return;
    }
    while (np<2048&&fgets(buff,sizeof(buff),fp)) {
        if (buff[0]=='%'||buff[0]=='#') continue;
        if (sscanf(buff,"%lf %lf %lf %s",&poss[np][0],&poss[np][1],&poss[np][2],
                   str)<4) continue;
        strncpy(stas[np],str,15); stas[np++][15]='\0';
    }
    fclose(fp);
    len=(int)strlen(rcv);
    for (i=0;i<np;i++) {
        if (strncmp(stas[i],rcv,len)) continue;
        for (j=0;j<3;j++) pos[j]=poss[i][j];
        pos[0]*=D2R; pos[1]*=D2R;
        return;
    }
    pos[0]=pos[1]=pos[2]=0.0;
}
/* compare observation data -------------------------------------------------*/
static int cmpobs(const void *p1, const void *p2)
{
    obsd_t *q1=(obsd_t *)p1,*q2=(obsd_t *)p2;
    double tt=timediff(q1->time,q2->time);
    if (fabs(tt)>DTTOL) return tt<0?-1:1;
    return q1->rcv!=q2->rcv?(int)q1->rcv-(int)q2->rcv:(int)q1->sat-(int)q2->sat;
}
/* sort observation data -------------------------------------------------------
* sort observation data by time, rcv, sat
* args   : obs_t *obs    IO     observation data
* return : number of epochs
*-----------------------------------------------------------------------------*/
extern int sortobs(obs_t *obs)
{
    int i,j,n;
    
    trace(3,"sortobs: nobs=%d\n",obs->n);
    
    if (obs->n<=0) return 0;
    qsort(obs->data,obs->n,sizeof(obsd_t),cmpobs);
    for (i=n=0;i<obs->n;i=j,n++) {
        for (j=i+1;j<obs->n;j++)
            if (timediff(obs->data[j].time,obs->data[i].time)>DTTOL) break;
    }
    return n;
}
/* compare satellite ephemerides ---------------------------------------------*/
static int cmpeph(const void *p1, const void *p2)
{
    eph_t *q1=(eph_t *)p1,*q2=(eph_t *)p2;
    
    return q1->sat!=q2->sat?q1->sat-q2->sat:(
           q1->ttr.time!=q2->ttr.time?(int)(q1->ttr.time-q2->ttr.time):
           (int)(q1->toe.time-q2->toe.time));
}
/* unique satellite ephemeris --------------------------------------------------
* sort satellite ephemerides by sat,ttr,toe and delete duplicated
* args   : nav_t *nav    IO     navigation messages
* return : none
*-----------------------------------------------------------------------------*/
extern void uniqeph(nav_t *nav)
{
    int i,j;
    
    trace(3,"uniqeph: nnav=%d\n",nav->n);
    
    if (nav->n<=0) return;
    qsort(nav->eph,nav->n,sizeof(eph_t),cmpeph);
    for (i=j=0;i<nav->n;i++) {
        if (nav->eph[i].sat!=nav->eph[j].sat||nav->eph[i].iode!=nav->eph[j].iode)
            nav->eph[++j]=nav->eph[i];
    }
    nav->n=j+1;
}
/* screen by time --------------------------------------------------------------
* screening by time start, time end, and time interval
* args   : gtime_t time  I      time
*          gtime_t ts    I      time start (ts.time==0:no screening by ts)
*          gtime_t te    I      time end   (te.time==0:no screening by te)
*          double  tint  I      time interval (s) (0.0:no screen by tint)
* return : 1:on condition, 0:not on condition
*-----------------------------------------------------------------------------*/
extern int screent(gtime_t time, gtime_t ts, gtime_t te, double tint)
{
    return (tint<=0.0||fmod(time2gpst(time,NULL)+DTTOL,tint)<=DTTOL*2.0)&&
           (ts.time==0||timediff(time,ts)>=-DTTOL)&&
           (te.time==0||timediff(time,te)<= DTTOL);
}
/* debug trace functions -----------------------------------------------------*/
#ifdef TRACE

static FILE *fp_trace=NULL;     /* file pointer of trace */
static int level_trace=1;       /* level of trace */
static unsigned int tick_trace=0; /* tick time at traceopen (ms) */

extern void traceopen(const char *file)
{
    if (!(fp_trace=fopen(file,"w"))) fp_trace=stdout;
    tick_trace=tickget();
}
extern void traceclose(void)
{
    fclose(fp_trace);
    fp_trace=NULL;
}
extern void tracelevel(int level)
{
    level_trace=level;
}
extern void trace(int level, const char *format, ...)
{
    va_list ap;
    
    if (!fp_trace||level>level_trace) return;
    fprintf(fp_trace,"%d ",level);
    va_start(ap,format); vfprintf(fp_trace,format,ap); va_end(ap);
    fflush(fp_trace);
}
extern void tracet(int level, const char *format, ...)
{
    va_list ap;
    
    if (!fp_trace||level>level_trace) return;
    fprintf(fp_trace,"%d %10.3f : ",level,(tickget()-tick_trace)/1000.0);
    va_start(ap,format); vfprintf(fp_trace,format,ap); va_end(ap);
    fflush(fp_trace);
}
extern void tracemat(int level, const double *A, int n, int m, int p, int q)
{
    if (!fp_trace||level>level_trace) return;
    matfprint(A,n,m,p,q,fp_trace); fflush(fp_trace);
}
extern void traceobs(int level, const obsd_t *obs, int n)
{
    char str[64],id[16];
    int i;
    
    if (!fp_trace||level>level_trace) return;
    for (i=0;i<n;i++) {
        time2str(obs[i].time,str,3);
        satno2id(obs[i].sat,id);
        fprintf(fp_trace," (%2d) %s %-3s rcv%d %13.3f %13.3f %13.3f %13.3f %d %d %2d %2d\n",
              i+1,str,id,obs[i].rcv,obs[i].L[0],obs[i].L[1],obs[i].P[0],
              obs[i].P[1],obs[i].LLI[0],obs[i].LLI[1],obs[i].SNR[0],obs[i].SNR[1]);
    }
    fflush(fp_trace);
}
extern void tracenav(int level, const nav_t *nav)
{
    char s1[64],s2[64],id[16];
    int i;
    
    if (!fp_trace||level>level_trace) return;
    for (i=0;i<nav->n;i++) {
        time2str(nav->eph[i].toe,s1,0);
        time2str(nav->eph[i].ttr,s2,0);
        satno2id(nav->eph[i].sat,id);
        fprintf(fp_trace,"(%2d) %-3s : %s %s %3d %3d %02x\n",i+1,
                id,s1,s2,nav->eph[i].iode,nav->eph[i].iodc,nav->eph[i].svh);
    }
    fprintf(fp_trace,"(ion) %9.4e %9.4e %9.4e %9.4e\n",nav->ion[0],nav->ion[1],
            nav->ion[2],nav->ion[3]);
    fprintf(fp_trace,"(ion) %9.4e %9.4e %9.4e %9.4e\n",nav->ion[4],nav->ion[5],
            nav->ion[6],nav->ion[7]);
}
extern void traceb(int level, const unsigned char *p, int n)
{
    int i;
    if (!fp_trace||level>level_trace) return;
    for (i=0;i<n;i++) fprintf(fp_trace,"%02X%s",*p++,i%8==7?" ":"");
    fprintf(fp_trace,"\n");
}
#else
extern void traceopen(const char *file) {}
extern void traceclose(void) {}
extern void tracelevel(int level) {}
extern void trace(int level, const char *format, ...) {}
extern void tracet(int level, const char *format, ...) {}
extern void tracemat(int level, const double *A, int n, int m, int p, int q) {}
extern void traceobs(int level, const obsd_t *obs, int n) {}
extern void tracenav(int level, const nav_t *nav) {}
extern void traceb(int level, const unsigned char *p, int n) {}

#endif /* TRACE */

/* execute command -------------------------------------------------------------
* execute command line by operating system shell
* args   : char   *cmd      I   command line
* return : execution status (0:ok,0>:error)
*-----------------------------------------------------------------------------*/
extern int execcmd(const char *cmd)
{
#ifdef WIN32
    PROCESS_INFORMATION info;
    STARTUPINFO si={0};
    DWORD stat;
    char cmds[1024];
    
    trace(3,"execcmd: cmd=%s\n",cmd);
    
    si.cb=sizeof(si);
    sprintf(cmds,"cmd /c %s",cmd);
    if (!CreateProcess(NULL,(LPTSTR)cmds,NULL,NULL,FALSE,CREATE_NO_WINDOW,NULL,
                       NULL,&si,&info)) return -1;
    WaitForSingleObject(info.hProcess,INFINITE);
    if (!GetExitCodeProcess(info.hProcess,&stat)) stat=-1;
    CloseHandle(info.hProcess);
    CloseHandle(info.hThread);
    return (int)stat;
#else
    trace(3,"execcmd: cmd=%s\n",cmd);
    
    return system(cmd);
#endif
}
/* expand file path ------------------------------------------------------------
* expand file path with wild-card (*) in file
* args   : char   *path     I   file path to expand (captal insensitive)
*          char   *paths    O   expanded file paths
*          int    nmax      I   max number of expanded file paths
* return : number of expanded file paths
*-----------------------------------------------------------------------------*/
extern int expath(const char *path, char *paths[], int nmax)
{
#ifdef WIN32
    WIN32_FIND_DATA file;
    HANDLE h;
    int i,n=0;
    char dir[1024]="",*p;
    
    trace(3,"expath : path=%s nmax=%d\n",path,nmax);
    
    if ((p=strrchr(path,'\\'))) {
        strncpy(dir,path,p-path+1); dir[p-path+1]='\0';
    }
    if ((h=FindFirstFile((LPCTSTR)path,&file))==INVALID_HANDLE_VALUE) {
        strcpy(paths[0],path);
        return 1;
    }
    sprintf(paths[n++],"%s%s",dir,file.cFileName);
    while (FindNextFile(h,&file)&&n<nmax) {
        if (file.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) continue;
        sprintf(paths[n++],"%s%s",dir,file.cFileName);
    }
    FindClose(h);
    
    for (i=0;i<n;i++) trace(3,"expath : path%d=%s\n",i,paths[i]);
    
    return n;
#else
    struct dirent *d;
    DIR *dp;
    int i,n=0;
    const char *file=path;
    char dir[1024]="",s1[1024],s2[1024],*p,*q;
    
    trace(3,"expath : path=%s nmax=%d\n",path,nmax);
    
    if ((p=strrchr(path,'/'))||(p=strrchr(path,'\\'))) {
        file=p+1; strncpy(dir,path,p-path+1); dir[p-path+1]='\0';
    }
    if (!(dp=opendir(*dir?dir:"."))) return 0;
    while ((d=readdir(dp))) {
        if (*(d->d_name)=='.') continue;
        sprintf(s1,"^%s$",d->d_name);
        sprintf(s2,"^%s$",file);
        for (p=s1;*p;p++) *p=(char)tolower(*p);
        for (p=s2;*p;p++) *p=(char)tolower(*p);
        for (p=s1,q=strtok(s2,"*");q;q=strtok(NULL,"*")) {
            if ((p=strstr(p,q))) p+=strlen(q); else break;
        }
        if (p&&n<nmax) sprintf(paths[n++],"%s%s",dir,d->d_name);
    }
    closedir(dp);
    
    for (i=0;i<n;i++) trace(3,"expath : path%d=%s\n",i,paths[i]);
    
    return n;
#endif
}
/* satellite ephemeris to satellite position/clock-bias ------------------------
* compute satellite position and clock-bias with satellite ephemeris
* args   : gtime_t time     I   transmission time by satellite clock
*                               (not true transmission time)
*          eph_t *eph       I   satellite ephemeris
*          double *rs       O   satellilte positions (ecef) (m)
*          double *dts      O   satellilte clock-bias (sec)
* return : none
* notes  : *dts includes TGD correction (P1 clock value)
*-----------------------------------------------------------------------------*/
extern void eph2pos(gtime_t time, const eph_t *eph, double *rs, double *dts)
{
    double tt,tc,tk,M,E,Ek,sinE,cosE,u,r,i,O,sin2u,cos2u,x,y,sinO,cosO,cosi;
    int n;
    
    if (!eph) { /* no ephemeris */
        rs[0]=rs[1]=rs[2]=dts[0]=0.0;
        return;
    }
    for (n=0,tt=0.0;n<2;n++) { /* transmission time correction */
        tc=timediff(time,eph->toc)-tt; tt=eph->f0+eph->f1*tc+eph->f2*tc*tc;
    }
    tk=timediff(time,eph->toe)-tt;
    M=eph->M0+(sqrt(MU/(eph->A*eph->A*eph->A))+eph->deln)*tk;
    for (E=M,sinE=Ek=0.0;fabs(E-Ek)>1E-12;) {
        Ek=E; sinE=sin(Ek); E=M+eph->e*sinE;
    }
    cosE=cos(E);
    u=atan2(sqrt(1.0-eph->e*eph->e)*sinE,cosE-eph->e)+eph->omg;
    r=eph->A*(1.0-eph->e*cosE);
    i=eph->i0+eph->idot*tk;
    sin2u=sin(2.0*u); cos2u=cos(2.0*u);
    u+=eph->cus*sin2u+eph->cuc*cos2u;
    r+=eph->crs*sin2u+eph->crc*cos2u;
    i+=eph->cis*sin2u+eph->cic*cos2u;
    O=eph->OMG0+(eph->OMGd-OMGE)*tk-OMGE*eph->toes;
    x=r*cos(u); y=r*sin(u); sinO=sin(O); cosO=cos(O); cosi=cos(i);
    rs[0]=x*cosO-y*cosi*sinO;
    rs[1]=x*sinO+y*cosi*cosO;
    rs[2]=y*sin(i);
    *dts=tt-2.0*sqrt(MU*eph->A)*eph->e*sinE/CLIGHT/CLIGHT-eph->tgd;
}
/* select ephememeris by time ------------------------------------------------*/
static eph_t *seleph(gtime_t time, int sat, const nav_t *nav)
{
    double tt,tmax,tmin;
    int i,j=-1;
    
    tmax=satsys(sat,NULL)==SYS_GPS?MAXDTOE+1.0:MAXDTOE_S;
    tmin=tmax+1.0;
    
    for (i=0;i<nav->n;i++) {
        if (nav->eph[i].sat!=sat) continue;
        if ((tt=fabs(timediff(nav->eph[i].toe,time)))>tmax) continue;
        if (nav->eph[i].svh!=0) {
            trace(2,"seleph : sat health bad (sat=%d svh=%x)\n",sat,nav->eph[i].svh);
            return NULL; /* satellite health bad */
        }
        if (tt<tmin) {j=i; tmin=tt;} /* toe closest to time */
    }
    return j<0?NULL:nav->eph+j;
}
/* select ephememeris by iode ------------------------------------------------*/
static eph_t *selephiode(gtime_t time, int sat, int iode, const nav_t *nav)
{
    int i;
    
    for (i=0;i<nav->n;i++) {
        if (nav->eph[i].sat!=sat||nav->eph[i].iode!=iode) continue;
        return nav->eph[i].svh!=0?NULL:nav->eph+i;
    }
    return NULL;
}
/* glonass orbit differential equations --------------------------------------*/
static void deq(const double *x, double *xdot, const double *acc)
{
    double a,b,r2=dot(x,x,3),r3=r2*sqrt(r2),omg2=OMGE_GLO*OMGE_GLO;
    
    /* GLONASS ICD A.3.1.2 with xdot[4],xdot[5] correction */
    a=1.5*J2_GLO*MU_GLO*RE_GLO*RE_GLO/r2/r3; /* 3/2*J2*myu*Ae^2/r^5 */
    b=5.0*x[2]*x[2]/r2;                      /* 5*z^2/r^2 */
    
    xdot[0]=x[3]; xdot[1]=x[4]; xdot[2]=x[5];
    xdot[3]=-MU_GLO*x[0]/r3-a*x[0]*(1.0-b)+omg2*x[0]+2.0*OMGE_GLO*x[4]+acc[0];
    xdot[4]=-MU_GLO*x[1]/r3-a*x[1]*(1.0-b)+omg2*x[1]-2.0*OMGE_GLO*x[3]+acc[1];
    xdot[5]=-MU_GLO*x[2]/r3-a*x[2]*(3.0-b)+acc[2];
}
/* glonass position/velocity by numerical integration ------------------------*/
static void glorbit(double t, double *x, const double *acc)
{
    double k1[6],k2[6],k3[6],k4[6],w[6];
    int i;
    
    deq(x,k1,acc); for (i=0;i<6;i++) w[i]=x[i]+k1[i]*t/2.0;
    deq(w,k2,acc); for (i=0;i<6;i++) w[i]=x[i]+k2[i]*t/2.0;
    deq(w,k3,acc); for (i=0;i<6;i++) w[i]=x[i]+k3[i]*t;
    deq(w,k4,acc);
    for (i=0;i<6;i++) x[i]+=(k1[i]+2.0*k2[i]+2.0*k3[i]+k4[i])*t/6.0;
}
/* glonass ephemeris to glonass position/clock-bias ----------------------------
* compute GLONASS position and clock-bias from glonass ephemeris
* args   : gtime_t time     I   trasmission time by satellite clock
*          geph_t *geph     I   glonass ephemeris
*          double *rs       O   glonass positions (ecef) (m)
*          double *dts      O   glonass clock-bias (sec)
* return : none
*-----------------------------------------------------------------------------*/
extern void geph2pos(gtime_t time, const geph_t *geph, double *rs, double *dts)
{
    double t,tt,x[6];
    int i;
    
    if (!geph) { /* no ephemeris */
        rs[0]=rs[1]=rs[2]=dts[0]=0.0;
        return;
    }
    t=timediff(time,geph->toe);
    for (i=0;i<2;i++) {
        t-=geph->taun+geph->gamn*t;
    }
    *dts=geph->taun+geph->gamn*t;
    
    for (i=0;i<3;i++) {
        x[i  ]=geph->pos[i];
        x[i+3]=geph->vel[i];
    }
    for (tt=t<0.0?-TSTEP:TSTEP;fabs(t)>1E-12;t-=tt) {
        if (fabs(t)<TSTEP) tt=t;
        glorbit(tt,x,geph->acc);
    }
    for (i=0;i<3;i++) rs[i]=x[i];
}
/* select glonass ephememeris by time ----------------------------------------*/
static geph_t *selgeph(gtime_t time, int sat, const nav_t *nav)
{
    double tt;
    int i;
    for (i=0;i<nav->ng;i++) {
        if (nav->geph[i].sat!=sat) continue;
        if (nav->geph[i].svh) {
            trace(2,"selgeph : sat health bad (sat=%d svh=%x)\n",sat,nav->geph[i].svh);
            return NULL; /* satellite health bad */
        }
        tt=timediff(time,nav->geph[i].toe);
        if (tt<-MAXDTOE_GLO||MAXDTOE_GLO<=tt) continue;
        return nav->geph+i;
    }
    return NULL;
}
/* satellite position/clock-bias ---------------------------------------------*/
static void satpos_(gtime_t time, const obsd_t *obs, int iode, const nav_t *nav,
                    double *rs, double *dts, double toff)
{
    eph_t *eph=NULL;
    geph_t *geph;
    gtime_t ts;
    
    if (obs->P[0]==0.0) {
        rs[0]=rs[1]=rs[2]=dts[0]=0.0;
        return;
    }
    /* transmission time by satellite clock */
    ts=timeadd(obs->time,-obs->P[0]/CLIGHT+toff);
    
    switch (satsys(obs->sat,NULL)) {
        case SYS_GPS:
        case SYS_GAL:
        case SYS_QZS:
            if (iode>=0) eph=selephiode(time,obs->sat,iode,nav);
            if (iode<0||!eph) eph=seleph(time,obs->sat,nav);
            eph2pos(ts,eph,rs,dts);
            break;
        case SYS_GLO:
            geph=selgeph(time,obs->sat,nav);
            geph2pos(ts,geph,rs,dts);
            break;
        default:
            rs[0]=rs[1]=rs[2]=dts[0]=0.0;
            break;
    }
}
/* satellite positions/clock-biases --------------------------------------------
* compute satellite positions and clock-biases from satellite ephemerides
* args   : gtime_t time     I   time to search ephemeris
*          obsd_t *obs      I   observation data records
*         (int    *iode     I   iodes)
*          int     n        I   number of observation data records
*          nav_t  *nav      I   navigation messages (sorted by transmssion time)
*          double *rs       O   satellilte positions (ecef) (m)
*          double *dts      O   satellilte clock-bias (sec)
* return : none
* notes  : rs[(0:2)+i*3],dts[i] = obs[i] satellite position/clock-bias
*          if no navigation messages, set 0.0 to rs[], dts[]
*-----------------------------------------------------------------------------*/
extern void satpos(gtime_t time, const obsd_t *obs, int n, const nav_t *nav,
                   double *rs, double *dts)
{
    satposiode(time,obs,NULL,n,nav,rs,dts);
}
extern void satposiode(gtime_t time, const obsd_t *obs, const int *iode,
                       int n, const nav_t *nav, double *rs, double *dts)
{
    int i;
    
    for (i=0;i<n&&i<MAXOBS;i++) {
        satpos_(time,obs+i,iode?iode[i]:-1,nav,rs+i*3,dts+i,0.0);
    }
}
/* satellite position/velocity/clock-bias/clock-drift --------------------------
* compute satellite position/velocities and clock-bias/drift from satellite
* ephemerides
* args   : gtime_t time     I   time to search ephemeris
*          obsd_t *obs      I   observation data records
*          int     n        I   number of observation data records
*          nav_t  *nav      I   navigation messages (sorted by transmssion time)
*          double *rs       O   satellilte position/velocity (ecef) (m,m/s)
*          double *dts      O   satellilte clock-bias/clock-drift (s,s/s)
* return : none
* notes  : rs[(0:5)+i*6],dts[(0:1)+i*2] = obs[i] satellite position/clock-bias
*          if no navigation messages, set 0.0 to vs[], ddts[]
*-----------------------------------------------------------------------------*/
extern void satposv(gtime_t time, const obsd_t *obs, int n, const nav_t *nav,
                    double *rs, double *dts)
{
    double rsd[3],dtsd,dt=1E-3; /* time increment for differential approx. */
    int i,j;
    
    for (i=0;i<n&&i<MAXOBS;i++) {
        satpos_(time,obs+i,-1,nav,rs+i*6,dts+i*2,0.0);
        satpos_(time,obs+i,-1,nav,rsd,&dtsd,dt);
        for (j=0;j<3;j++) rs[j+3+i*6]=(rsd[j]-rs[j+i*6])/dt;
        dts[1+i*2]=(dtsd-dts[i*2])/dt;
    }
}
/* geometric distance ----------------------------------------------------------
* compute geometric distance and receiver-to-satellite unit vector
* args   : double *rs       I   satellilte position (ecef at transmission) (m)
*          double *rr       I   receiver position (ecef at reception) (m)
*          double *e        O   line-of-sight vector (ecef)
* return : geometric distance (m) (0>:error/no satellite position)
* notes  : distance includes sagnac effect correction
*-----------------------------------------------------------------------------*/
extern double geodist(const double *rs, const double *rr, double *e)
{
    double r;
    int i;
    
    if (norm(rs,3)<RE) return -1.0;
    for (i=0;i<3;i++) e[i]=rs[i]-rr[i];
    r=norm(e,3);
    for (i=0;i<3;i++) e[i]/=r;
    return r+OMGE*(rs[0]*rr[1]-rs[1]*rr[0])/CLIGHT;
}
/* satellite azimuth/elevation angle -------------------------------------------
* compute satellite azimuth/elevation angle
* args   : double *pos      I   geodetic position {lat,lon,h} (rad,m)
*          double *e        I   receiver-to-satellilte unit vevtor (ecef)
*          double *azel     IO  azimuth/elevation {az,el} (rad) (NULL: no output)
* return : elevation angle (rad)
*-----------------------------------------------------------------------------*/
extern double satazel(const double *pos, const double *e, double *azel)
{
    double az=0.0,el=PI/2.0,enu[3];
    
    if (pos[2]>-RE) {
        ecef2enu(pos,e,enu);
        az=dot(enu,enu,2)<1E-12?0.0:atan2(enu[0],enu[1]);
        el=asin(enu[2]);
    }
    if (azel) {azel[0]=az; azel[1]=el;}
    return el;
}
/* compute dops ----------------------------------------------------------------
* compute DOP (dilution of precision)
* args   : int    ns        I   number of satellites
*          double *azel     I   satellite azimuth/elevation angle (rad)
*          double elmin     I   elevation cutoff angle (rad)
*          double *dop      O   DOPs {GDOP,PDOP,HDOP,VDOP}
* return : none
* notes  : dop[0]-[3] return 0 in case of dop computation error
*-----------------------------------------------------------------------------*/
#define SQRT(x)     ((x)<0.0?0.0:sqrt(x))

extern void dops(int ns, const double *azel, double elmin, double *dop)
{
    double H[4*MAXSAT],Q[16],*Hi;
    int i,n;
    
    for (i=0;i<4;i++) dop[i]=0.0;
    for (i=n=0;i<ns&&i<MAXSAT;i++) {
        if (azel[1+i*2]<elmin||azel[1+i*2]<=0.0) continue;
        Hi=H+4*n++;
        Hi[0]=cos(azel[1+i*2])*sin(azel[i*2]);
        Hi[1]=cos(azel[1+i*2])*cos(azel[i*2]);
        Hi[2]=sin(azel[1+i*2]); Hi[3]=1.0;
    }
    if (n<4) return;
    matmul("NT",4,4,n,1.0,H,H,0.0,Q);
    if (matinv(Q,4)) {
        return;
    }
    dop[0]=SQRT(Q[0]+Q[5]+Q[10]+Q[15]);
    dop[1]=SQRT(Q[0]+Q[5]+Q[10]);
    dop[2]=SQRT(Q[0]+Q[5]);
    dop[3]=SQRT(Q[10]);
}
/* ionosphere model ------------------------------------------------------------
* compute ionospheric delay by broadcast ionosphere model (klobuchar model)
* args   : gtime_t t        I   gtime_t struct
*          double *ion      I   iono model parameters {a0,a1,a2,a3,b0,b1,b2,b3}
*          double *pos      I   receiver position {lat,lon,h} (rad,m)
*          double *azel     I   azimuth/elevation angle {az,el} (rad)
* return : ionospheric delay (L1) (m)
*-----------------------------------------------------------------------------*/
extern double ionmodel(gtime_t t, const double *ion, const double *pos,
                       const double *azel)
{
    const double ion_default[]={ /* 2004/1/1 */
        0.1118E-07,-0.7451E-08,-0.5961E-07, 0.1192E-06,
        0.1167E+06,-0.2294E+06,-0.1311E+06, 0.1049E+07
    };
    double tt,f,psi,phi,lam,amp,per,x;
    int week;
    
    if (pos[2]<-1E3||azel[1]<=0) return 0.0;
    if (norm(ion,8)<=0.0) ion=ion_default;
    psi=0.0137/(azel[1]/PI+0.11)-0.022;
    phi=pos[0]/PI+psi*cos(azel[0]);
    phi=phi<-0.416?-0.416:(phi>0.416?0.416:phi);
    lam=pos[1]/PI+psi*sin(azel[0])/cos(phi*PI);
    phi+=0.064*cos((lam-1.617)*PI);
    amp=ion[0]+phi*(ion[1]+phi*(ion[2]+phi*ion[3]));
    per=ion[4]+phi*(ion[5]+phi*(ion[6]+phi*ion[7]));
    amp=amp<0.0?0.0:amp; per=per<72000.0?72000.0:per;
    f=1.0+16.0*pow(0.53-azel[1]/PI,3.0);
    tt=4.32E4*lam+time2gpst(t,&week);
    tt-=floor(tt/86400.0)*86400.0; /* 0<=tt<86400 */
    x=2.0*PI*(tt-50400.0)/per;
    return CLIGHT*f*(fabs(x)<1.57?5E-9+amp*(1.0+x*x*(-0.5+x*x/24.0)):5E-9);
}
/* ionosphere mapping function -------------------------------------------------
* compute ionospheric delay mapping function by single layer model
* args   : double *pos      I   receiver position {lat,lon,h} (rad,m)
*          double *azel     I   azimuth/elevation angle {az,el} (rad)
* return : ionospheric mapping function
*-----------------------------------------------------------------------------*/
extern double ionmapf(const double *pos, const double *azel)
{
    if (pos[2]>=HION) return 1.0;
    return 1.0/cos(asin((RE+pos[2])/(RE+HION)*sin(PI/2.0-azel[1])));
}
/* troposphere model -----------------------------------------------------------
* compute tropospheric delay by standard atmosphere and saastamoinen model
* args   : double *pos      I   receiver position {lat,lon,h} (rad,m)
*          double *azel     I   azimuth/elevation angle {az,el} (rad)
*          double humi      I   relative humidity
* return : tropospheric delay (m)
*-----------------------------------------------------------------------------*/
extern double tropmodel(const double *pos, const double *azel, double humi)
{
    double dtrop,h,pres,temp,e,tz;
    
    if (pos[2]<-100.0||1E4<pos[2]||azel[1]<=0) return 0.0;
    h=pos[2]<0.0?0.0:pos[2];
    pres=1013.25*pow(1.0-2.2557E-5*h,5.257);
    temp=15.0-6.5E-3*h;
    e=6.108*pow(10.0,7.5*temp/(237.3+temp))*humi;
    tz=tan(PI/2.0-azel[1]);
    dtrop=0.002277/sin(azel[1])*(pres+(1255.0/(temp+273.15)+0.05)*e-tz*tz);
    return dtrop;
}
/* troposphere mapping function (NMF) ------------------------------------------
* compute tropospheric mapping function by NMF
* args   : gtime_t t        I   time
*          double *pos      I   receiver position {lat,lon,h} (rad,m)
*          double *azel     I   azimuth/elevation angle {az,el} (rad)
*          double *mapfw    IO  wet mapping function (NULL: not output)
* return : dry mapping function
* note   : reference
*          A.E.Niell, Global mapping functions for the atmosphere delay at
*          radio wavelengths, J.Geophys.Res. 1996
*-----------------------------------------------------------------------------*/
static double interpc(const double coef[], double lat)
{
    int i=(int)(lat/15.0);
    if (i<1) return coef[0]; else if (i>4) return coef[4];
    return coef[i-1]*(1.0-lat/15.0+i)+coef[i]*(lat/15.0-i);
}
static double mapf(double el, double a, double b, double c)
{
    double sinel=sin(el);
    return (1.0+a/(1.0+b/(1.0+c)))/(sinel+(a/(sinel+b/(sinel+c))));
}
extern double tropmapf(gtime_t time, const double pos[], const double azel[],
                       double *mapfw)
{
    static const double coef[][5]={
        {1.2769934E-3,1.2683230E-3,1.2465397E-3,1.2196049E-3,1.2045996E-3},
        {2.9153695E-3,2.9152299E-3,2.9288445E-3,2.9022565E-3,2.9024912E-3},
        {62.610505E-3,62.837393E-3,63.721774E-3,63.824265E-3,64.258455E-3},
        {0.0000000E-0,1.2709626E-5,2.6523662E-5,3.4000452E-5,4.1202191E-5},
        {0.0000000E-0,2.1414979E-5,3.0160779E-5,7.2562722E-5,11.723375E-5},
        {0.0000000E-0,9.0128400E-5,4.3497037E-5,84.795348E-5,170.37206E-5},
        {5.8021897E-4,5.6794847E-4,5.8118019E-4,5.9727542E-4,6.1641693E-4},
        {1.4275268E-3,1.5138625E-3,1.4572752E-3,1.5007428E-3,1.7599082E-3},
        {4.3472961E-2,4.6729510E-2,4.3908931E-2,4.4626982E-2,5.4736038E-2}
    };
    static const double aht=2.53E-5,bht=5.49E-3,cht=1.14E-3;
    double c,ah,bh,ch,aw,bw,cw,dmdh,lat;
    
    trace(3,"tropmapf: pos=%.6f %.6f %.1f azel=%.1f %.1f\n",pos[0]*R2D,
          pos[1]*R2D,pos[2],azel[0]*R2D,azel[1]*R2D);
    
    if (azel[1]<=0.0) {if (mapfw) *mapfw=0.0; return 0.0;}
    c=cos(2.0*PI*(time2doy(time)-28.0)/365.25);
    lat=fabs(pos[0])*R2D;
    ah=interpc(coef[0],lat)+interpc(coef[3],lat)*c;
    bh=interpc(coef[1],lat)+interpc(coef[4],lat)*c;
    ch=interpc(coef[2],lat)+interpc(coef[5],lat)*c;
    aw=interpc(coef[6],lat); bw=interpc(coef[7],lat); cw=interpc(coef[8],lat);
    dmdh=1.0/sin(azel[1])-mapf(azel[1],aht,bht,cht)*(pos[0]>=0.0?1.0:-1.0);
    if (mapfw) *mapfw=mapf(azel[1],aw,bw,cw);
    return mapf(azel[1],ah,bh,ch)+dmdh*pos[2]/1E3; 
}

/* antenna model ---------------------------------------------------------------
* compute antenna offset by antenna phase center parameters
* args   : pcv_t *pcv       I   antenna phase center parameters
*          double *azel     I   azimuth/elevation angle {az,el} (rad)
*          double *dant     O   range offsets for each frequency (m)
* return : none
*-----------------------------------------------------------------------------*/
static double interpvar(double el, const double *var)
{
    double a=el/(5.0*D2R);
    int i=(int)a;
    if (i<0) return var[0]; else if (i>=18) return var[18];
    return var[i]*(a-i)+var[i+1]*(1.0-a+i);
}
extern void antmodel(const pcv_t *pcv, const double *azel, double *dant)
{
    int i,j;
    double off[3],enu[3],cosel=cos(azel[1]);
    
    trace(3,"antmodel: azel=%6.1f %4.1f\n",azel[0]*R2D,azel[1]*R2D);
    
    enu[0]=sin(azel[0])*cosel; enu[1]=cos(azel[0])*cosel; enu[2]=sin(azel[1]);
    for (i=0;i<NFREQ;i++) {
        for (j=0;j<3;j++) off[j]=pcv->ecc[j]+pcv->off[i][j];
        dant[i]=-dot(off,enu,3)+interpvar(azel[1],pcv->var[i]);
    }
}
/* carrier smoothing -----------------------------------------------------------
* carrier smoothing by Hatch filter
* args   : obs_t  *obs      IO  raw observation data/smoothed observation data
*          int    ns        I   smoothing window size (epochs)
* return : none
*-----------------------------------------------------------------------------*/
extern void csmooth(obs_t *obs, int ns)
{
    double Ps[2][MAXSAT][NFREQ]={{{0}}},Lp[2][MAXSAT][NFREQ]={{{0}}},dcp;
    int i,j,s,r,n[2][MAXSAT][NFREQ]={{{0}}};
    obsd_t *p;
    
    trace(3,"csmooth: nobs=%d,ns=%d\n",obs->n,ns);
    
    for (i=0;i<obs->n;i++) {
        p=&obs->data[i]; s=p->sat; r=p->rcv;
        for (j=0;j<NFREQ;j++) {
            if (s<=0||MAXSAT<s||r<=0||2<r) continue;
            if (p->P[j]==0.0||p->L[j]==0.0) continue;
            if (p->LLI[j]) n[r-1][s-1][j]=0;
            if (n[r-1][s-1][j]==0) Ps[r-1][s-1][j]=p->P[j];
            else {
                dcp=lam[j]*(p->L[j]-Lp[r-1][s-1][j]);
                Ps[r-1][s-1][j]=p->P[j]/ns+(Ps[r-1][s-1][j]+dcp)*(ns-1)/ns;
            }
            if (++n[r-1][s-1][j]<ns) p->P[j]=0.0; else p->P[j]=Ps[r-1][s-1][j];
            Lp[r-1][s-1][j]=p->L[j];
        }
    }
}
/* pseudorange measurement error variance ------------------------------------*/
static double varerr(double el, double dion)
{
    double a2=ERRA*ERRA,b2=ERRB*ERRB,c2=ERRI*ERRI,sinel=sin(el);
    return a2+b2/sinel/sinel+c2*dion*dion;
}
/* pseudorange residuals -----------------------------------------------------*/
static int residuals(const obsd_t *obs, int n, double *rs, const double *dts,
                     const double *x, const double *ion, double elmin,
                     double snrmin, double *v, double *H, double *azel, int *vsat)
{
    int i,j,nv=0;
    double r,dion,sig,pos[3],e[3];
    
    ecef2pos(x,pos); /* receiver geodetic postion */ 
    
    for (i=0;i<n&&i<MAXOBS;i++) {
        azel[i*2]=azel[1+i*2]=0.0; vsat[i]=0;
        
        /* pseudorange/geometric distance/satellite azimuth/elevation angle */
        if (obs[i].P[0]==0.0||(r=geodist(rs+i*3,x,e))<0.0) continue;
        if (satazel(pos,e,azel+i*2)<elmin) continue;
        if (snrmin>0.0&&0.0<obs[i].SNR[0]&&obs[i].SNR[0]<snrmin) continue;
        vsat[i]=1;
        
        /* pseudorange model */
        dion=ionmodel(obs[i].time,ion,pos,azel+i*2);
        r+=x[3]-CLIGHT*dts[i]+dion+tropmodel(pos,azel+i*2,0.5);
        
        /* measurement error standard deviation (m) */
        sig=sqrt(varerr(azel[1+i*2],dion));
        
        /* design matrix (normalized) */
        for (j=0;j<4;j++) H[j+nv*4]=(j<3?-e[j]:1.0)/sig;
        
        /* residuals (normalized) */
        v[nv++]=(obs[i].P[0]-r)/sig;
    }
    return nv;
}
/* validate solution ---------------------------------------------------------*/
static int valsol(const obsd_t *obs, const double *azel, int n, const double *v,
                  int nv, const double *Q)
{
    int i;
    
    for (i=0;i<n;i++) {
        if (azel[1+i*2]>=0.0) continue;
        trace(1,"pntpos : elevation error (sat=%d el=%.1f)\n",obs[i].sat,azel[1+i*2]*R2D);
        return -1;
    }
    if (nv>4&&dot(v,v,nv)>chisqr[nv-5]) {
        trace(1,"pntpos : validation error (ns=%d cs=%.1f)\n",nv,dot(v,v,nv));
        return -1;
    }
    if (Q[0]+Q[5]+Q[10]+Q[15]>MAXGDOP*MAXGDOP) {
        trace(1,"pntpos : gdop error (ns=%d gdop=%.1f)\n",nv,sqrt(Q[0]+Q[5]+Q[10]+Q[15]));
        return -1;
    }
    return 0;
}
/* single-point positioning ----------------------------------------------------
* compute receiver position and clock-bias by single-point positioning
* args   : obsd_t *obs      I   observation data records
*          int    n         I   number of observation data records
*          nav_t  *nav      I   navigation messages
*          double elmin     I   elevation cutoff angle (rad)
*          double snrmin    I   snr mask (dBHz)
*          double *rr       IO  initial/estimated receiver position (ecef) (m)
*          double *Qr       O   estimated position covarience (3 x 3)
*          double *dtr      O   estimated receiver clock-bias (sec)
*          double *azel     O   satellite azimuth/elevation angle (rad)
*          int    *vsat     O   valid satellite flag
* return : number of valid satellites(0:error)
*-----------------------------------------------------------------------------*/
extern int pntpos(const obsd_t *obs, int n, const nav_t *nav, double elmin,
                  double snrmin, double *rr, double *Qr, double *dtr,
                  double *azel, int *vsat)
{
    int i,j,nv,info=-1;
    double x[4],dx[4],Q[16],*v,*H,*rs,*dts;
    
    trace(3,"pntpos : nobs=%d,nnav=%d\n",n,nav->n);
    
    if (n<4) {
        trace(1,"pntpos : number of sats error (ns=%d)\n",n);
        return -1;
    }
    v=mat(n,1); H=mat(4,n); rs=mat(3,n); dts=mat(1,n);
    
    /* satellite positions/clock-biases */
    satpos(obs[0].time,obs,n,nav,rs,dts);
    
    /* initial position/clock-bias */
    for (i=0;i<3;i++) x[i]=rr[i]; x[3]=0.0;
    
    for (i=0;i<MAXITR;i++) {
        /* residuals */
        if ((nv=residuals(obs,n,rs,dts,x,nav->ion,elmin,snrmin,v,H,azel,vsat))<4) {
            trace(1,"pntpos : number of valid sats error (ns=%d)\n",nv);
            break;
        }
        /* least square estimation */
        if ((info=lsq(H,v,4,nv,dx,Q))) {
            trace(1,"pntpos : lsq error (info=%d)\n",info);
            break;
        }
        for (j=0;j<4;j++) x[j]+=dx[j];
        
        if (norm(dx,4)<1E-4) break;
    }
    if (i>=MAXITR) {
        trace(1,"pntpos : iteration divergent (n=%d)\n",i);
    }
    /* validate solution */
    if (!info&&i<MAXITR&&!valsol(obs,azel,n,v,nv,Q)) {
        for (i=0;i<3;i++) {rr[i]=x[i]; for (j=0;j<3;j++) Qr[i+j*3]=Q[i+j*4];}
        *dtr=x[3]/CLIGHT;
    }
    else nv=0;
    free(v); free(H); free(rs); free(dts);
    return nv;
}
/* doppler residuals ---------------------------------------------------------*/
static int resdop(const obsd_t *obs, int n, const double *rs, const double *dts,
                  const double *rr, const double *x, const double *azel,
                  const int *vsat, double *v, double *H)
{
    int i,j,nv=0;
    double y,pos[3],E[9],a[3],e[3],vs[3];
    
    trace(3,"resdop  : nobs=%d\n",n);
    
    ecef2pos(rr,pos); xyz2enu(pos,E);
    
    for (i=0;i<n&&i<MAXOBS;i++) {
        if (!vsat[i]||obs[i].D[0]==0.0||norm(rs+3+i*6,3)<=0.0) continue;
        
        /* line-of-sight vector (ecef) */
        a[0]=sin(azel[i*2])*cos(azel[1+i*2]);
        a[1]=cos(azel[i*2])*cos(azel[1+i*2]);
        a[2]=sin(azel[1+i*2]);
        matmul("TN",3,1,3,1.0,E,a,0.0,e);
        
        /* range-rate model */
        for (j=0;j<3;j++) vs[j]=rs[j+3+i*6]-x[j];
        y=dot(vs,e,3)+OMGE/CLIGHT*(rs[4+i*6]*rr[0]+rs[1+i*6]*x[0]-
                                   rs[3+i*6]*rr[1]-rs[i*6]*x[1]);
        y+=x[3]-CLIGHT*dts[1+i*2];
        
        /* design matrix */
        for (j=0;j<4;j++) H[j+nv*4]=j<3?-e[j]:1.0;
        
        /* residuals */
        v[nv++]=-lam[0]*obs[i].D[0]-y; 
    }
    return nv;
}
/* velocity estimation by single-point positioning -----------------------------
* compute receiver position/velocity and clock-bias/drift
* args   : obsd_t *obs      I   observation data records
*          int    n         I   number of observation data records
*          nav_t  *nav      I   navigation messages
*          double *rr       I   receiver position (ecef) (m)
*          double *azel     I   satellite azimuth/elevation angle (rad)
*          int    *vsat     I   valid satellite flag
*          double *vr       O   estimated velocity (ecef) (m/s) (3 x 1)
*          double *Qv       O   estimated velocity covarience (3 x 3)
*          double *ddtr     O   estimated receiver clock-drift (s/s)
* return : number of valid satellites(<0:error)
*-----------------------------------------------------------------------------*/
extern int pntvel(const obsd_t *obs, int n, const nav_t *nav, const double *rr,
                  const double *azel, const int *vsat, double *vr, double *Qv,
                  double *ddtr)
{
    int i,j,nv,info=-1;
    double x[4],dx[4],Q[16],*v,*H,*rs,*dts;
    
    trace(3,"pntvel : nobs=%d,nnav=%d\n",n,nav->n);
    
    v=mat(n,1); H=mat(4,n); rs=mat(6,n); dts=mat(2,n);
    
    /* satellite position/velocity/clock-biase/clock-drift */
    satposv(obs[0].time,obs,n,nav,rs,dts);
    
    /* initial velocity/clock-drift */
    for (i=0;i<4;i++) x[i]=0.0;
    
    for (i=0;i<MAXITR;i++) {
        
        /* doppler residuals */
        if ((nv=resdop(obs,n,rs,dts,rr,x,azel,vsat,v,H))<4) {
            trace(1,"pntvel : number of valid sats error (ns=%d)\n",nv);
            break;
        }
        /* least square estimation */
        if ((info=lsq(H,v,4,nv,dx,Q))) {
            trace(1,"pntvel : lsq error (info=%d)\n",info);
            nv=0;
            break;
        }
        for (j=0;j<4;j++) x[j]+=dx[j];
        if (norm(dx,4)<1E-6) break;
    }
    if (i>=MAXITR) {
        trace(1,"pntvel : iteration divergent (n=%d)\n",i);
        nv=0;
    }
    if (nv>=4&&!info&&i<MAXITR) {
        for (i=0;i<3;i++) {
            vr[i]=x[i];
            for (j=0;j<3;j++) Qv[i+j*3]=Q[i+j*4];
        }
        *ddtr=x[3]/CLIGHT;
    }
    free(v); free(H); free(rs); free(dts);
    return nv;
}
